@@ -37,54 +37,6 @@ Logged as bugzilla bug 319752.]]]
 As a means to keep things simple for now, regions can only exist on a single
 line.  That is, a region cannot cross line boundaries.
 
-The main entry points into this module are as follows:
-
-    o init/shutdown:  for initializing and shutting down this module
-
-    o Line:           a class whose instances maintain an ordered list
-                      of Regions
-
-    o Region:         a class whose instances represent an area of text;
-                      subclasses of Regions include Component, Text, etc.
-
-    o Component:      a Region that will present the label for an
-                      accessible component and also cause the component
-                      to be activated when the user presses a cursor
-                      routing key associated with the component.
-
-    o Text:           a Region that presents the text of an accessible
-                      text object and will also route the text caret
-                      appopriately when the user presses a cursor routing
-                      key.
-
-    o ReviewComponent: a special Component used for flat review; keeps
-                      track of the flat review Zone associated with the
-                      region.
-
-    o ReviewText:     a special Component used for flat review; keeps
-                      track of the Zone associated with the region and
-                      also keeps track of where the review cursor is.
-
-    o clear:          clears the logical structure, but keeps the Braille
-                      display as is (see refresh)
-
-    o refresh:        repaints the logical structure on the Braille display
-                      clipping content based upon the viewport setting
-
-    o addLine:        appends a Line instance to the logical structure
-
-    o setFocus:       utility routine to set the cursor to a region
-
-Fields include:
-
-    o endIsShowing:    if True, the last character of a Line is showing
-                       somewhere on the display.
-    o beginningIsShowing: if True, the first character of a Line is showing
-                       on the display.
-    o cursorCell:      a 1-based offset saying which cell on the display
-                       has the cursor.  cursorCell=1 is the leftmost
-                       cell.  cursorCell=0 means the cursor is not showing.
-
 After initialization, a typical use of this module would be as follows:
 
     braille.clear()
@@ -104,8 +56,10 @@ Braille display.
 
 import atspi
 import brl
+import brlmon
 import debug
 import eventsynthesizer
+import settings
 
 from orca_i18n import _                          # for gettext support
 from rolenames import getShortBrailleForRoleName # localized role names
@@ -115,6 +69,10 @@ from rolenames import getLongBrailleForRoleName  # localized role names
 #
 _initialized = False
 
+# The braille monitor
+#
+monitor = None
+
 # Each of these maps to BrlAPI's brldefs.h file.
 #
 CMD_NOOP              = 0x00
@@ -465,7 +423,12 @@ class Line:
         for region in self.regions:
             if region == _regionWithFocus:
                 focusOffset = len(string)
-            string += region.string
+            if region.string:
+                # [[[TODO: WDW - HACK: Replace unicode ellipses with "..."
+                # The ultimate solution is to get i18n support into
+                # BrlTTY.]]]
+                #
+                string += region.string.replace("\342\200\246", "...")
 
         return [string, focusOffset]
 
@@ -623,6 +586,7 @@ def refresh(panToCursor=True, targetCursorCell=0):
     global endIsShowing
     global beginningIsShowing
     global cursorCell
+    global monitor
 
     if len(_lines) == 0:
         brl.writeText(0, "")
@@ -676,12 +640,46 @@ def refresh(panToCursor=True, targetCursorCell=0):
 
     debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" \
                   % (string[startPos:endPos], cursorCell))
-
+  
     brl.writeText(cursorCell, string[startPos:endPos])
 
+    if settings.getSetting(settings.USE_BRAILLE_MONITOR, False):
+	if not monitor:	
+	    monitor = brlmon.BrlMon(_displaySize[0])
+	    monitor.show_all()
+        monitor.writeText(cursorCell, string[startPos:endPos])
+
     beginningIsShowing = startPos == 0
     endIsShowing = endPos >= len(string)
 
+def displayRegionsWithFocus(regions, focusedRegion):
+    """Displays a list of regions on a single line, setting focus to the 
+       specified region.
+
+    Arguments:
+    - regions: a list of regions to display
+    - focusedRegion: the focused region (must be in regions)
+    """
+
+    clear()
+    line = Line()
+    for item in regions:
+        line.addRegion(item)
+    addLine(line)
+    setFocus(focusedRegion)
+    refresh()
+
+def displayRegions(regions, indexOfFocusRegion=0):
+    """Displays a list of regions on a single line, setting focus to the 
+       specified region.
+
+    Arguments:
+    - regions: a list of regions to display
+    - indexOfFocusRegion: which region in the list should receive focus,
+                          or the first one if this is not specified
+    """
+    displayRegionsWithFocus(regions, regions[indexOfFocusRegion])
+
 def displayMessage(message, cursor=-1):
     """Displays a single line, setting the cursor to the given position,
     ensuring that the cursor is in view.
