@@ -110,6 +110,10 @@ class SpeechServer(speechserver.SpeechServer):
         if not isInitialized:
             isInitialized = driver.driverInit()
         if not isInitialized:
+            try:
+                driver.unref()
+            except:
+                pass
             driver = None
 	return driver
 
@@ -176,19 +180,18 @@ class SpeechServer(speechserver.SpeechServer):
 		    break
 
 	if not gserver:
-	    return server
+	    return None
 
 	try:
 	    driver = SpeechServer.__activateDriver(gserver.iid)
 	    server = SpeechServer(driver, gserver.iid)
 	except:
-	    if info:
-		return server
 	    for s in gservers:
 		try:
 		    driver = SpeechServer.__activateDriver(s.iid)
 	            if driver:
 		        server = SpeechServer(driver, s.iid)
+                        break
 		except:
 		    debug.printException(debug.LEVEL_WARNING)
 		    pass
@@ -494,8 +497,17 @@ class SpeechServer(speechserver.SpeechServer):
             except:
                 pass
         else:
+            # Replace ellipses (both manual and unicode) with "dot dot dot"
+            #
             text = text.replace("...", _(" dot dot dot"), 1)
+            text = text.replace("\342\200\246",  _(" dot dot dot"), 1)
 
+        # [[[TODO: HACK - We're going to add whitespace after "["
+        # because DECtalk wants to interpret these as inline DECtalk
+        # commands.]]]
+        #
+        text = text.replace("[", "[ ")
+        
         # Send the text to the GNOME Speech speaker
         #
         debug.println(debug.LEVEL_INFO, "SPEECH OUTPUT: '" + text + "'")
