@@ -25,6 +25,7 @@ import orca.orca as orca
 import orca.braille as braille
 import orca.speech as speech
 import orca.settings as settings
+import orca.util as util
 
 from orca.orca_i18n import _ # for gettext support
 
@@ -99,26 +100,6 @@ class Script(default.Script):
         print "<<<<============================>>>>"
 
 
-    def isDesiredFocusedItem(self, obj, rolesList):
-        """Called to determine if the given object and it's hierarchy of
-           parent objects, each have the desired roles.
-
-        Arguments:
-        - rolesList: the list of desired roles for the components and the
-          hierarchy of its parents.
-
-        Returns True if all roles match.
-        """
-
-        current = obj
-        for i in range(0, len(rolesList)):
-            if (current == None) or (current.role != rolesList[i]):
-                return False
-            current = current.parent
-
-        return True
-
-
     def readPageTab(self, tab):
         """Speak/Braille the given page tab. The speech verbosity is set
            to VERBOSITY_LEVEL_BRIEF for this operation and then restored
@@ -216,7 +197,7 @@ class Script(default.Script):
         rolesList = [rolenames.ROLE_TEXT, \
                      rolenames.ROLE_PANEL, \
                      rolenames.ROLE_UNKNOWN]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail view: current message pane: " \
                       + "individual lines of text.")
@@ -252,7 +233,7 @@ class Script(default.Script):
                      rolenames.ROLE_PANEL, \
                      rolenames.ROLE_TABLE_CELL]
         if (self.readTableCellRow == True) \
-            and (self.isDesiredFocusedItem(event.source, rolesList)):
+            and (util.isDesiredFocusedItem(event.source, rolesList)):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail view: current message pane: " \
                       + "standard mail header lines.")
@@ -285,9 +266,10 @@ class Script(default.Script):
         #
         # Check if the focus is in the message header list. If this focus
         # event is for a different row that the last time we got a similar
-        # focus event, we want to speak all of the tables cells (and their
-        # headers) in the current highlighted message. (The role is only 
-        # spoken/brailled for the table cell that currently has focus).
+        # focus event, we want to speak all of the tables cells (and the
+        # header for the one that currently has focus) in the current 
+        # highlighted message. (The role is only spoken/brailled for the 
+        # table cell that currently has focus).
         #
         # If this focus event is just for a different table cell on the same
         # row as last time, then we just speak the current cell (and its
@@ -302,7 +284,7 @@ class Script(default.Script):
         rolesList = [rolenames.ROLE_TABLE_CELL, \
                      rolenames.ROLE_TREE_TABLE]
         if (self.readTableCellRow == True) \
-            and (self.isDesiredFocusedItem(event.source, rolesList)):
+            and (util.isDesiredFocusedItem(event.source, rolesList)):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail view: message header list.")
 
@@ -343,7 +325,7 @@ class Script(default.Script):
 
             if orca.locusOfFocus.role != rolenames.ROLE_TABLE_CELL:
                 speakAll = True
-                message = "There are %d messages in this mail folder" % \
+                message = "%d messages" % \
                     parent.table.nRows
                 brailleRegions.append(braille.Region(message))
                 speech.speak(message)
@@ -374,8 +356,8 @@ class Script(default.Script):
 
                     if toRead:
                         # Speak/braille the column header for this table cell
-                        # (unless it's a checkbox).
-                        if not checkbox:
+                        # if it has focus (unless it's a checkbox).
+                        if not checkbox and verbose:
                             settings.brailleVerbosityLevel = \
                                 settings.VERBOSITY_LEVEL_BRIEF
                             settings.speechVerbosityLevel = \
@@ -452,7 +434,7 @@ class Script(default.Script):
 
         rolesList = [rolenames.ROLE_CALENDAR_EVENT, \
                      rolenames.ROLE_CALENDAR_VIEW]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - calendar view: day view: " \
                       + "tabbing to day with appts.")
@@ -521,7 +503,7 @@ class Script(default.Script):
         rolesList = [rolenames.ROLE_UNKNOWN, \
                      rolenames.ROLE_TABLE, \
                      rolenames.ROLE_CALENDAR_VIEW]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - calendar view: day view: " \
                       + "moving with arrow keys.")
@@ -596,7 +578,7 @@ class Script(default.Script):
                      rolenames.ROLE_TABLE, \
                      rolenames.ROLE_UNKNOWN, \
                      rolenames.ROLE_SCROLL_PANE]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - preferences dialog: " \
                       + "table cell in options list.")
@@ -632,7 +614,7 @@ class Script(default.Script):
                      rolenames.ROLE_FILLER, \
                      rolenames.ROLE_FILLER, \
                      rolenames.ROLE_DIALOG]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail insert attachment dialog: " \
                       + "unlabelled button.")
@@ -660,7 +642,7 @@ class Script(default.Script):
                      rolenames.ROLE_PANEL, \
                      rolenames.ROLE_PANEL, \
                      rolenames.ROLE_SCROLL_PANE]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail compose window: " \
                       + "message area.")
@@ -685,7 +667,7 @@ class Script(default.Script):
                     rolenames.ROLE_PANEL, \
                     rolenames.ROLE_FILLER, \
                     rolenames.ROLE_DIALOG]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - spell checking dialog.")
 
@@ -728,7 +710,7 @@ class Script(default.Script):
                 # to either side of it).
                 #
                 for i in range(0, len(allTokens)):
-                    if allTokens[i] == badWord:
+                    if allTokens[i].startswith(badWord):
                         min = i - 5
                         if min < 0:
                             min = 0
@@ -736,8 +718,8 @@ class Script(default.Script):
                         if max > (len(allTokens) - 1):
                             max = len(allTokens) - 1
 
-                        utterances = [_("Misspelled word is "), badWord, \
-                                  _(" Context is ")] + allTokens[min:max]
+                        utterances = [_("Misspelled word: "), badWord, \
+                                  _(" Context is ")] + allTokens[min:max+1]
 
                         # Turn the list of utterances into a string.
                         text = " ".join(utterances)
@@ -766,7 +748,7 @@ class Script(default.Script):
                     rolenames.ROLE_TABLE_CELL, \
                     rolenames.ROLE_TABLE, \
                     rolenames.ROLE_PANEL]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if util.isDesiredFocusedItem(event.source, rolesList):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail message area attachments.")
 
