@@ -18,6 +18,8 @@
 # Boston, MA 02111-1307, USA.
 
 import orca.default as default
+import orca.util as util
+import orca.orca as orca
 
 ########################################################################
 #                                                                      #
@@ -35,3 +37,17 @@ class Script(default.Script):
         """
 
         default.Script.__init__(self, app)
+
+    def onWindowActivated(self, event):
+        # Sets the context to the top level window first, so we can
+        # get information about it the window we just moved to.
+        #
+        orca.setLocusOfFocus(event, event.source)
+
+        # Now we find the focused object and set the locus of focus to it.
+        #
+	obj = util.findFocusedObject(self.app)
+	if obj:
+	    orca.setLocusOfFocus(event, obj)
+        else:
+	    default.Script.onWindowActivated(self, event)
