@@ -21,7 +21,6 @@
 as its speech server, or it can feel free to create one of its own.
 """
 
-import pdb
 import debug
 import settings
 
@@ -61,8 +60,6 @@ def init():
     if __speechserver:
         return
 
-    #pdb.set_trace()
-
     # First, find the factory module to use.  We will
     # allow the user to give their own factory module,
     # thus we look first in the global name space, and
@@ -104,6 +101,10 @@ def __resolveACSS(acss=None):
         voices = settings.getSetting(settings.VOICES, None)
         return voices[settings.DEFAULT_VOICE]
 
+def sayAll(utteranceIterator, progressCallback):
+    if __speechserver:
+        __speechserver.sayAll(utteranceIterator, progressCallback)
+    
 def speak(text, acss=None):
     if __speechserver:
         __speechserver.speak(text, __resolveACSS(acss))
@@ -132,6 +133,11 @@ def shutdown():
         __speechserver.shutdown()
         __speechserver = None
 
+def reset(text=None, acss=None):
+    global __speechserver
+    if __speechserver:
+        __speechserver.reset(text, acss)
+
 def testNoSettingsInit():
     init()
     speak("testing")
